/*
 * grzesiostat.c
 *
 * Created: 2016-06-17 13:25:18
 *  Author: dsosnowski
 */ 

/*
 * kubostat.c
 *
 * Created: 2016-06-09 10:17:47
 *  Author: dsosnowski
 */ 


#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include "hard.h"
#include "ee_mem.h"
#include "wdog.h"

//-----------------------------------------------------
#define NUM_OF_INPUTS				11
#define MASTER_MODE_CHANNEL			9
#define MASTER_RST_CHANNEL			10
#define LAST_INPUT_CHANNEL			4

#define MAIN_DELAY					20
#define MAIN_STARTUP				10
#define INPUT_THRESHOLD_ON_CYCLE	4

#define NEW_DELAY					6
#define MASTER_PULSE				8

#define MASTER_MODE_SELECTOR		0
#define MASTER_MODE_SWITCH			1
#define SWITCH_MODE_MONO			0
#define SWITCH_MODE_BISTABLE		1
#define SELECT_MODE_DEPEND			0
#define SELECT_MODE_INDEPEND		1


//-----------------------------------------------------
char InputProc(void);
char InputGet(char channel);
void OutputProc(char state);
void LedProc(char state);


//-----------------------------------------------------
struct {
	char action;
	char state;
	char config;
	char startup;

	char new_action;
	char new_state;
	char new_delay;
	
	char master_pulse;
	char master_mode;
} system;

char inputs[(NUM_OF_INPUTS + 1)];

//-----------------------------------------------------
int main(void)
{
	char prev;
	OUTS_DDR |= (1<<OUT1_PIN | 1<<OUT2_PIN | 1<<OUT3_PIN | 1<<OUT4_PIN);
	
	INPUTS_PORT |= (1<<INPUT2_PIN | 1<<INPUT3_PIN| 1<<INPUT4_PIN);
	INPUT_1_PORT |= (1<<INPUT1_PIN | 1<<CFG_PK3_PIN	| 1<<CFG_PK4_PIN);
	CFGS_PORT |= (1<<CFG_PK1_PIN | 1<<CFG_PK2_PIN | 1<<CFG_SEL_PIN);
	
	LED12_DDR |= (1<<LED1_PIN | 1<<LED2_PIN);
	LED34_DDR |= (1<<LED3_PIN | 1<<LED4_PIN);
	
		
	_delay_ms(100);
	WdtSet(7);

	system.startup = MAIN_STARTUP;
	
    while(1)
    {
		//----------------------------------------------------------------
		system.action = InputProc();
		
		if (system.startup) {
			system.startup--;
			system.action = 0;
		}
				
		//----------------------------------------------------------------
		if (InputGet(MASTER_MODE_CHANNEL) == MASTER_MODE_SWITCH){
			if (InputGet(LAST_INPUT_CHANNEL + 1) == SWITCH_MODE_MONO) {
				if (InputGet(1) == 0) system.state &= ~(1<<0);
			}
			if (InputGet(LAST_INPUT_CHANNEL + 2) == SWITCH_MODE_MONO) {
				if (InputGet(2) == 0) system.state &= ~(1<<1);
			}
			if (InputGet(LAST_INPUT_CHANNEL + 3) == SWITCH_MODE_MONO) {
				if (InputGet(3) == 0) system.state &= ~(1<<2);
			}
			if (InputGet(LAST_INPUT_CHANNEL + 4) == SWITCH_MODE_MONO) {
				if (InputGet(4) == 0) system.state &= ~(1<<3);
			}
		}
		
		//----------------------------------------------------------------
		if (system.master_pulse == 0) {
			if (system.action == MASTER_RST_CHANNEL) {
				if (InputGet(MASTER_MODE_CHANNEL) == MASTER_MODE_SELECTOR){
					if (InputGet(LAST_INPUT_CHANNEL + 1) == SELECT_MODE_DEPEND) system.state &= ~(1<<0);
					if (InputGet(LAST_INPUT_CHANNEL + 2) == SELECT_MODE_DEPEND) system.state &= ~(1<<1);
					if (InputGet(LAST_INPUT_CHANNEL + 3) == SELECT_MODE_DEPEND) system.state &= ~(1<<2);
					if (InputGet(LAST_INPUT_CHANNEL + 4) == SELECT_MODE_DEPEND) system.state &= ~(1<<3);
				}
			}
		}
		
		//----------------------------------------------------------------
		if ((system.action > 0) && (system.action <= LAST_INPUT_CHANNEL)) {
			system.config = (system.action + LAST_INPUT_CHANNEL);
			system.new_action = (1<<(system.action - 1));
			
			prev = system.state;
			
			if (InputGet(MASTER_MODE_CHANNEL) == MASTER_MODE_SWITCH){
				if (InputGet(system.config) == SWITCH_MODE_MONO) {
					system.state |= system.new_action;
				
				//if (InputGet(system.config) == SWITCH_MODE_BISTABLE) {
				} else {
					if (system.state & system.new_action) {
						system.state &= ~(system.new_action);
					} else {
						system.state |= system.new_action;
					}
				}

			//if (InputGet(MASTER_MODE_CHANNEL) == MASTER_MODE_SELECTOR){
			} else {
				if (InputGet(system.config) == SELECT_MODE_DEPEND) {
					//system.state = 0;
					if (InputGet(LAST_INPUT_CHANNEL + 1) == SELECT_MODE_DEPEND) system.state &= ~(1<<0);
					if (InputGet(LAST_INPUT_CHANNEL + 2) == SELECT_MODE_DEPEND) system.state &= ~(1<<1);
					if (InputGet(LAST_INPUT_CHANNEL + 3) == SELECT_MODE_DEPEND) system.state &= ~(1<<2);
					if (InputGet(LAST_INPUT_CHANNEL + 4) == SELECT_MODE_DEPEND) system.state &= ~(1<<3);
					
					system.new_state = system.state;
					system.new_state |= system.new_action;
					system.new_delay = NEW_DELAY;
					system.master_pulse = MASTER_PULSE;
					if (system.new_state == prev) system.new_delay = 1;

				//if (InputGet(system.config) == SELECT_MODE_INDEPEND) {
				} else {
					if ((system.state & system.new_action) > 0) {
						system.state &= ~(system.new_action);
					} else {
						system.state |= system.new_action;
					}
				}				
			}		
			system.action = 0;
		}
			
		//-------------------------------------
		if (system.master_pulse > 0) {
			MASTER_RST_SET;
			system.master_pulse--;
		} else {
			MASTER_RST_OFF;
		}

		//-------------------------------------
		if (system.new_delay > 0) {
			system.new_delay--;
			if (system.new_delay == 0) {
				system.state = system.new_state;
			}
		}
		
		//-------------------------------------
		OutputProc(system.state);
		LedProc(system.state);			

		_delay_ms(MAIN_DELAY);
		WdtReset();
    }
}


//-----------------------------------------------------
char InputProc(void) {
	char channel;
	char state;
	char result = 0;
	
	for(channel=1; channel<(NUM_OF_INPUTS + 1); channel++){
		state = 1;
		if (channel == 1) state = INPUT_1_PIN & (1<<INPUT1_PIN);
		if (channel == 2) state = INPUTS_PIN & (1<<INPUT2_PIN);
		if (channel == 3) state = INPUTS_PIN & (1<<INPUT3_PIN);
		if (channel == 4) state = INPUTS_PIN & (1<<INPUT4_PIN);
		if (channel == 5) state = CFGS_PIN & (1<<CFG_PK1_PIN);
		if (channel == 6) state = CFGS_PIN & (1<<CFG_PK2_PIN);
		if (channel == 7) state = CFG4_PIN & (1<<CFG_PK3_PIN);
		if (channel == 8) state = CFG4_PIN & (1<<CFG_PK4_PIN);
		if (channel == 9) state = MASTER_MODE_GET;
		if (channel == 10) state = MASTER_RST_GET;
			
		if (state == 0){
			if (inputs[channel] < 0xFF) {
				inputs[channel]++;
				if (inputs[channel] == INPUT_THRESHOLD_ON_CYCLE) result = channel;
			}
		} else {
			inputs[channel] = 0;
		}
	}
	return result;
}

//-----------------------------------------------------
char InputGet(char channel) {
	if (inputs[channel] >= INPUT_THRESHOLD_ON_CYCLE) return 1;
	return 0;
}

//-----------------------------------------------------
void OutputProc(char state) {	
	if (state & (1<<0))	
		OUTS_PORT |= (1<<OUT1_PIN);
	else 
		OUTS_PORT &= ~(1<<OUT1_PIN);
		
	if (state & (1<<1))
		OUTS_PORT |= (1<<OUT2_PIN);
	else
		OUTS_PORT &= ~(1<<OUT2_PIN);

	if (state & (1<<2))
		OUTS_PORT |= (1<<OUT3_PIN);
	else
		OUTS_PORT &= ~(1<<OUT3_PIN);

	if (state & (1<<3))
		OUTS_PORT |= (1<<OUT4_PIN);
	else
		OUTS_PORT &= ~(1<<OUT4_PIN);
}


//-----------------------------------------------------
void LedProc(char state) {
	if (state & (1<<0))
	LED12_PORT &= ~(1<<LED1_PIN);
	else
	LED12_PORT |= (1<<LED1_PIN);
	
	if (state & (1<<1))
	LED12_PORT &= ~(1<<LED2_PIN);
	else
	LED12_PORT |= (1<<LED2_PIN);

	if (state & (1<<2))
	LED34_PORT &= ~(1<<LED3_PIN);
	else
	LED34_PORT |= (1<<LED3_PIN);

	if (state & (1<<3))
	LED34_PORT &= ~(1<<LED4_PIN);
	else
	LED34_PORT |= (1<<LED4_PIN);
}